#include <stdio.h>
#include <stdlib.h>
#include "types.h"

// U/V Data 	Flag 	Description
// Bit 7 	C 	Composite Sync Flag (active low). Refer to the timing diagrams.
// Bit 6 	F 	Field Flag. F = "0" for odd-number fields (Fields 1, 3, . . . ), F = "1" for even-number fields (Fields 2, 4, . . . ).
// Bit 5 	V 	Vertical Sync Flag (active low). Refer to timing diagrams.
// Bit 4 	H 	Horizontal Sync Flag (active low). Refer to timing diagrams.
// Bit 3 	B 	Burst Flag (active low). This signal indicates the location of the color burst. It is active once every line.
// Bit 2 	K 	Burst Blank Flag (active low). When active, color burst of a line should be blanked out.
// Bit 1 	N 	NTSC/PAL and M-PAL Mode Flag. N = "0" for NTSC mode, N ="1" for PAL or M-PAL mode.
// Bit 0 	I 	Interlace/Non-Interface Mode Flag. 1 = "0" for interlace mode, I = "1" for non-interlace mode.


/* defines section */
#define REQ_COMMAND_LINE_PRAM    2
#define USAGE                    "Usage:   read_port capture.bin \n"
#define IMAGE_FILE               1
#define SCAN_LENGTH              24
#define HALF_SCAN_LENGTH         28
#define NULL_SCAN_LENGTH         16
#define HALF_NULL_SCAN_LENGTH_1  20
#define HALF_NULL_SCAN_LENGTH_2  24
#define BURST_BLANK_LENGTH       16
#define HALF_BURST_BLANK_LENGTH  20    
#define VERT_SYNCH_LENGTH        20
#define HALF_VERT_SYNCH_LENGTH   24 

/* run this program using the console pauser or add your own getch, system("pause") or input loop */

int main(int argc, char *argv[])
{
	FILE *pfile = NULL;
    long file_size = 0;
    U8 *pbuffer = NULL;
    U8 *pworking = NULL;
	U32 result = 0; 
	U32 line_count = 0;
	/* normal scan line */
	const char scan_line[SCAN_LENGTH] = "\x00\x2F\x00\x3F\x00\xBF\x00\x0A\x00\xB7\x00\x1E\x00\xBF\x00\x41\x10\x80\x02\x7F\x00\xBF\x00\x33";
	/* half scan line  */
	const char half_scan_line[HALF_SCAN_LENGTH] = "\x00\x2F\x00\x3F\x00\xBF\x00\x0A\x00\xB7\x00\x1E\x00\xBF\x00\x41\x10\x10\x00\x0F\x10\x80\x00\xF3\x00\xBF\x01\xAF";
    /* full null scan line */
    const char null_scan_line[NULL_SCAN_LENGTH] = "\x00\x2F\x00\x3F\x00\xBF\x00\x0A\x00\xB7\x00\x1E\x00\xBF\x02\xF5";
    /* half null scan line */
    const char half_null_scan_line_1[HALF_NULL_SCAN_LENGTH_1]  = "\x00\x2F\x00\x3F\x00\xBF\x00\x0A\x00\xB7\x00\x1E\x00\xBF\x01\x45\x00\xBB\x01\xAF";
    const char half_null_scan_line_2[HALF_NULL_SCAN_LENGTH_2]  = "\x00\x2F\x00\x3F\x00\xBF\x00\x0A\x00\xB7\x00\x1E\x00\xBF\x01\x45\x10\x80\x01\x7B\x00\xBF\x00\x33";
    
    /* burst blank line */
    const char burst_blank_line[BURST_BLANK_LENGTH] = "\x00\x2B\x00\x3F\x00\xBB\x00\x0A\x00\xB3\x00\x1E\x00\xBB\x02\xF5";
    /* half burst blank line */
    const char half_burst_blank_line_1[HALF_BURST_BLANK_LENGTH] = "\x00\x2B\x00\x3F\x00\xBB\x00\x0A\x00\xB3\x00\x1E\x00\xBB\x01\x45\x00\x1B\x01\xAF";
    const char half_burst_blank_line_2[HALF_BURST_BLANK_LENGTH] = "\x00\x2B\x00\x3F\x00\xBB\x00\x0A\x00\xB3\x00\x1E\x00\xBB\x01\x45\x00\xBF\x01\xAF";
	/* vert synch line */
    const char vert_sync_line[VERT_SYNCH_LENGTH] = "\x00\x9B\x00\x3F\x00\x0B\x00\x3F\x00\x1B\x00\x0A\x00\x13\x00\x1E\x00\x1B\x02\xB5";
    /* half vert synch line */
    const char half_vert_sync_line[HALF_VERT_SYNCH_LENGTH] = "\x00\x9B\x00\x3F\x00\x0B\x00\x3F\x00\x1B\x00\x0A\x00\x13\x00\x1E\x00\x1B\x01\x45\x00\xBB\x01\xAF";

	U32 count = 0;
	U32 i;
	
    /* check we have the required files */
    if(argc < REQ_COMMAND_LINE_PRAM)
    {
        printf("Incorrect number of args \n");
        printf(USAGE);	
        return 1;              
    }
    
      /* open labels file */
    pfile = fopen(argv[IMAGE_FILE],"rb");
    
     /* check file opened ok */
    if(pfile == NULL)
    {
        printf("cant open file %s \n",argv[IMAGE_FILE]);	
        return 1;             
    }
    
    /* check file actully holds data */
    fseek(pfile,0, SEEK_END);
    file_size = ftell(pfile);
    rewind(pfile);
    
    if(file_size == 0)
    {         
        printf("file %s is empty\n",argv[IMAGE_FILE]);	
        return 1;      
    }
    
    /* get memory to copy file too */
    pbuffer = (U8 *) calloc (1, file_size);
    
    if(pbuffer == NULL)
    {
        printf("cant malloc memory to hold file ! \n"); 
        system("PAUSE");	
        return 1;              
    }
  
    /* copy file into memory */
    result = fread(pbuffer, 1, file_size, pfile);
  
    if(result != file_size)
    {
        printf("file read error %x != %x\n",result,file_size);
        free(pbuffer);
        system("PAUSE");	
        return 1;          
    }

    fclose(pfile); 

    pworking = pbuffer;

    /* find number of scan lines before blanking starts */
    for(i = 0; i < file_size; i += SCAN_LENGTH)
    {
        /* check each scan line is correct */
        if(0 != memcmp((pworking), &scan_line,SCAN_LENGTH))
        {
    	    break;
        }
        
        pworking += SCAN_LENGTH;
        count++;
    }
    
    printf("number of horz scan lines is %3d finishs at 0x%x\n", count, i);
    
    count = 0;
    
    /* find number of half scan lines */
	for(; i < file_size; i += HALF_SCAN_LENGTH)
    {
        /* check each half scan line is correct */
        if(0 != memcmp((pworking), &half_scan_line, HALF_SCAN_LENGTH))
        {
    	    break;
        }
        
        pworking += HALF_SCAN_LENGTH;
        count++;
    }
    
    printf("number of half scan lines is %3d finishs at 0x%x\n", count, i);
    
    count = 0;
    
    /* find number of null scan lines */
	for(; i < file_size; i += NULL_SCAN_LENGTH)
    {
        /* check each null scan line is correct */
        if(0 != memcmp((pworking), &null_scan_line, NULL_SCAN_LENGTH))
        {
    	    break;
        }
        
        pworking += NULL_SCAN_LENGTH;
        count++;
    }
    
    printf("number of null scan lines is %3d finishs at 0x%x\n", count, i);
     
    count = 0; 
    
	/* find number of half null scan lines */
	for(; i < file_size; i += HALF_NULL_SCAN_LENGTH_1)
    {
        /* check each half null scan line is correct */
        if(0 != memcmp((pworking), &half_null_scan_line_1, HALF_NULL_SCAN_LENGTH_1))
        {
    	    break;
        }
        
        pworking += HALF_NULL_SCAN_LENGTH_1;
        count++;
    } 
	 
	printf("number of half null scan lines is %3d finishs at 0x%x\n", count, i);
	
	count = 0; 
    
	/* find number of burst blank lines */
	for(; i < file_size; i += BURST_BLANK_LENGTH)
    {
        /* check each burst blank line is correct */
        if(0 != memcmp((pworking), &burst_blank_line,BURST_BLANK_LENGTH))
        {
    	    break;
        }
        
        pworking += BURST_BLANK_LENGTH;
        count++;
    } 
	 
	printf("number of burst blank lines is %3d finishs at 0x%x\n", count, i);
	
	count = 0; 
    
	/* find number of half burst blank lines */
	for(; i < file_size; i += HALF_BURST_BLANK_LENGTH)
    {
        /* check each half burst blank line is correct */
        if(0 != memcmp((pworking), &half_burst_blank_line_1,HALF_BURST_BLANK_LENGTH))
        {
    	    break;
        }
        
        pworking += HALF_BURST_BLANK_LENGTH;
        count++;
    } 
	 
	printf("number of half burst blank lines is %3d finishs at 0x%x\n", count, i);
	    
	count = 0; 
    
	/* find number of vert synch lines */
	for(; i < file_size; i += VERT_SYNCH_LENGTH)
    {
        /* check each vert synch line is correct */
        if(0 != memcmp((pworking), &vert_sync_line,VERT_SYNCH_LENGTH))
        {
    	    break;
        }
        
        pworking += VERT_SYNCH_LENGTH;
        count++;
    } 
	 
	printf("number of vert synch lines is %3d finishs at 0x%x\n", count, i);
	        
	count = 0; 
    
	/* find number of vert synch lines */
	for(; i < file_size; i += HALF_VERT_SYNCH_LENGTH)
    {
        /* check each vert synch line is correct */
        if(0 != memcmp((pworking), &half_vert_sync_line,HALF_VERT_SYNCH_LENGTH))
        {
    	    break;
        }
        
        pworking += HALF_VERT_SYNCH_LENGTH;
        count++;
    } 
	 
	printf("number of half vert synch lines is %3d finishs at 0x%x\n", count, i);
	
	count = 0;
	
	/* find number of burst blank lines */
	for(; i < file_size; i += BURST_BLANK_LENGTH)
    {
        /* check each burst blank line is correct */
        if(0 != memcmp((pworking), &burst_blank_line,BURST_BLANK_LENGTH))
        {
    	    break;
        }
        
        pworking += BURST_BLANK_LENGTH;
        count++;
    } 
	 
	printf("number of burst blank lines is %3d finishs at 0x%x\n", count, i);
	
    count = 0;
	
	/* find number of half burst blank lines */
	for(; i < file_size; i += HALF_BURST_BLANK_LENGTH)
    {
        /* check each half burst blank line is correct */
        if(0 != memcmp((pworking), &half_burst_blank_line_2,HALF_BURST_BLANK_LENGTH))
        {
    	    break;
        }
        
        pworking += HALF_BURST_BLANK_LENGTH;
        count++;
    } 
	 
	printf("number of half burst blank lines is %3d finishs at 0x%x\n", count, i);
	
	count = 0;
	
    /* find number of null scan lines */
	for(; i < file_size; i += NULL_SCAN_LENGTH)
    {
        /* check each null scan line is correct */
        if(0 != memcmp((pworking), &null_scan_line, NULL_SCAN_LENGTH))
        {
    	    break;
        }
        
        pworking += NULL_SCAN_LENGTH;
        count++;
    }
    
    printf("number of null scan lines is %3d finishs at 0x%x\n", count, i);
    
    count = 0; 
    
	/* find number of half null scan lines */
	for(; i < file_size; i += HALF_NULL_SCAN_LENGTH_2)
    {
        /* check each half null scan line is correct */
        if(0 != memcmp((pworking), &half_null_scan_line_2, HALF_NULL_SCAN_LENGTH_2))
        {
    	    break;
        }
        
        pworking += HALF_NULL_SCAN_LENGTH_2;
        count++;
    } 
	 
	printf("number of half null scan lines is %3d finishs at 0x%x\n", count, i);
	count = 0; 
	
	 /* find number of scan lines before blanking starts */
    for(; i < file_size; i += SCAN_LENGTH)
    {
        /* check each scan line is correct */
        if(0 != memcmp((pworking), &scan_line,SCAN_LENGTH))
        {
    	    break;
        }
        
        pworking += SCAN_LENGTH;
        count++;
    }
    
    printf("number of horz scan lines is %3d finishs at 0x%x\n", count, i);
    
    /* free the memory after use */
    free(pbuffer);
	
	return 0;
}
